"""
A2UI Templates

Pre-built UI templates for common use cases.
"""

from praisonaiagents.ui.a2ui.templates.chat import ChatTemplate
from praisonaiagents.ui.a2ui.templates.list import ListTemplate
from praisonaiagents.ui.a2ui.templates.form import FormTemplate
from praisonaiagents.ui.a2ui.templates.dashboard import DashboardTemplate

__all__ = [
    "ChatTemplate",
    "ListTemplate",
    "FormTemplate",
    "DashboardTemplate",
]
